// -*- C++ -*-
/*!
 * @file  ConvertToMotor.cpp
 * @brief Convert PS4 Contrller Value
 * @date $Date$
 *
 * $Id$
 */

#include "ConvertToMotor.h"

// Module specification
// <rtc-template block="module_spec">
static const char* converttomotor_spec[] =
  {
    "implementation_id", "ConvertToMotor",
    "type_name",         "ConvertToMotor",
    "description",       "Convert PS4 Contrller Value",
    "version",           "1.0.0",
    "vendor",            "FSK",
    "category",          "Converter",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.ServoMotor1ChangeValue", "1",
    "conf.default.ServoMotor1LimitMax", "180",
    "conf.default.ServoMotor1LimitMin", "0",
    "conf.default.ServoMotor2ChangeValue", "1",
    "conf.default.ServoMotor2LimitMax", "180",
    "conf.default.ServoMotor2LimitMin", "0",
    "conf.default.ServoMotor3ChangeValue", "1",
    "conf.default.ServoMotor3LimitMax", "180",
    "conf.default.ServoMotor3LimitMin", "0",

    // Widget
    "conf.__widget__.ServoMotor1ChangeValue", "slider.0.1",
    "conf.__widget__.ServoMotor1LimitMax", "slider.1",
    "conf.__widget__.ServoMotor1LimitMin", "slider.1",
    "conf.__widget__.ServoMotor2ChangeValue", "slider.0.1",
    "conf.__widget__.ServoMotor2LimitMax", "slider.1",
    "conf.__widget__.ServoMotor2LimitMin", "slider.1",
    "conf.__widget__.ServoMotor3ChangeValue", "slider.0.1",
    "conf.__widget__.ServoMotor3LimitMax", "slider.1",
    "conf.__widget__.ServoMotor3LimitMin", "slider.1",
    // Constraints
    "conf.__constraints__.ServoMotor1ChangeValue", "1<=x<=10",
    "conf.__constraints__.ServoMotor1LimitMax", "0<=x<=180",
    "conf.__constraints__.ServoMotor1LimitMin", "0<=x<=180",
    "conf.__constraints__.ServoMotor2ChangeValue", "1<=x<=10",
    "conf.__constraints__.ServoMotor2LimitMax", "0<=x<=180",
    "conf.__constraints__.ServoMotor2LimitMin", "0<=x<=180",
    "conf.__constraints__.ServoMotor3ChangeValue", "1<=x<=10",
    "conf.__constraints__.ServoMotor3LimitMax", "0<=x<=180",
    "conf.__constraints__.ServoMotor3LimitMin", "0<=x<=180",

    "conf.__type__.ServoMotor1ChangeValue", "float",
    "conf.__type__.ServoMotor1LimitMax", "int",
    "conf.__type__.ServoMotor1LimitMin", "int",
    "conf.__type__.ServoMotor2ChangeValue", "float",
    "conf.__type__.ServoMotor2LimitMax", "int",
    "conf.__type__.ServoMotor2LimitMin", "int",
    "conf.__type__.ServoMotor3ChangeValue", "float",
    "conf.__type__.ServoMotor3LimitMax", "int",
    "conf.__type__.ServoMotor3LimitMin", "int",

    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ConvertToMotor::ConvertToMotor(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_Controller_TypeIn("Controller_Type", m_Controller_Type),
    m_ButtonIn("Button", m_Button),
    m_AnalogIn("Analog", m_Analog),
    m_ServoMotorValue1Out("ServoMotor1", m_ServoMotorValue1),
    m_ServoMotorValue2Out("ServoMotor2", m_ServoMotorValue2),
    m_ServoMotorValue3Out("ServoMotor3", m_ServoMotorValue3)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
ConvertToMotor::~ConvertToMotor()
{
}



RTC::ReturnCode_t ConvertToMotor::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("Controller_Type", m_Controller_TypeIn);
  addInPort("Button", m_ButtonIn);
  addInPort("Analog", m_AnalogIn);
  
  // Set OutPort buffer
  addOutPort("ServoMotor1", m_ServoMotorValue1Out);
  addOutPort("ServoMotor2", m_ServoMotorValue2Out);
  addOutPort("ServoMotor3", m_ServoMotorValue3Out);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("ServoMotor1ChangeValue", m_ServoMotor1ChangeValue, "1");
  bindParameter("ServoMotor1LimitMax", m_ServoMotor1LimitMax, "180");
  bindParameter("ServoMotor1LimitMin", m_ServoMotor1LimitMin, "0");
  bindParameter("ServoMotor2ChangeValue", m_ServoMotor2ChangeValue, "1");
  bindParameter("ServoMotor2LimitMax", m_ServoMotor2LimitMax, "180");
  bindParameter("ServoMotor2LimitMin", m_ServoMotor2LimitMin, "0");
  bindParameter("ServoMotor3ChangeValue", m_ServoMotor3ChangeValue, "1");
  bindParameter("ServoMotor3LimitMax", m_ServoMotor3LimitMax, "180");
  bindParameter("ServoMotor3LimitMin", m_ServoMotor3LimitMin, "0");
  // </rtc-template>
  button_input = 0;
  long f = 1;
  for (int i = 0; i < 14; i++)
  {
	  binary_number[i] = f << i;
	  std::cout << "bin num[" << i << "]:" << binary_number[i] << std::endl;
	  button[i] = 0;
  }
  m_ServoMotorValue1.data = 90;
  m_ServoMotorValue2.data = 90;
  m_ServoMotorValue3.data = 90;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ConvertToMotor::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ConvertToMotor::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ConvertToMotor::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ConvertToMotor::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "onActivated" << std::endl;
	for (int i = 0; i < 14; i++)button[i] = 0;

	butonFlag = false;
	AnalogFlag = false;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ConvertToMotor::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "onDeactivated" << std::endl;

	std::cout << "ServoMotor1:" << m_ServoMotorValue1.data << " ServoMotor2:" << m_ServoMotorValue2.data << " ServoMotor3:" << m_ServoMotorValue3.data << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ConvertToMotor::onExecute(RTC::UniqueId ec_id)
{
	AnalogFlag = false;
	butonFlag = false;

	if (m_Controller_TypeIn.isNew())
	{
		m_Controller_TypeIn.read();
		std::cout << m_Controller_Type.data << std::endl;
	}
	if (m_ButtonIn.isNew())//PS4̃{^Ă邩mF
	{
		m_ButtonIn.read();
		button_input = m_Button.data;
		std::cout << "button_input:" << button_input << "b:";
		for (int i = 0; i < 14; i++)
		{
			if (button_input >= binary_number[13 - i]){//ǂ̃{^Ă邩mF
				button[13 - i] = 1;
				button_input = button_input - binary_number[13 - i];
			}
			else
			{
				button[13 - i] = 0;
			}
			std::cout << button[13 - i];
		}
		std::cout << std::endl;
		butonFlag = true;
	}
	if (m_AnalogIn.isNew())
	{

		std::cout << "Analog:";
		m_AnalogIn.read();
		for (int i = 0; i < 8; i++){
			Analog[i] = m_Analog.data[i];
			std::cout << Analog[i] << ",";
		}
		std::cout << std::endl;
		AnalogFlag = true;


	}
	ServoMotorOut();
	std::cout << "Smotor1:" << m_ServoMotorValue1.data << " Smotor2:" << m_ServoMotorValue2.data << " Smotor3:" << m_ServoMotorValue3.data << std::endl;


  return RTC::RTC_OK;
}
void ConvertToMotor::ServoMotorOut()
{

	if (AnalogFlag == true)
	{
		if (Analog[2]> 0){//XeBbNɌXĂƂ
			m_ServoMotorValue2.data += (int)(Analog[2] * m_ServoMotor2ChangeValue);//px
			if (m_ServoMotorValue2.data > m_ServoMotor2LimitMax)
				m_ServoMotorValue2.data = m_ServoMotor2LimitMax;
		}
		else{//XeBbNEɌXĂƂ
			m_ServoMotorValue2.data += (int)(Analog[2] * m_ServoMotor2ChangeValue);//px
			if (m_ServoMotorValue2.data < m_ServoMotor2LimitMin)
				m_ServoMotorValue2.data = m_ServoMotor2LimitMin;
		}
		if (Analog[3]> 0){//XeBbNɌXĂƂ
			m_ServoMotorValue3.data += (int)(Analog[3] * m_ServoMotor3ChangeValue);
			if (m_ServoMotorValue3.data > m_ServoMotor3LimitMax)
				m_ServoMotorValue3.data = m_ServoMotor3LimitMax;
		}
		else{//XeBbNɌXĂƂ
			m_ServoMotorValue3.data += (int)(Analog[3] * m_ServoMotor3ChangeValue);
			if (m_ServoMotorValue3.data < m_ServoMotor3LimitMin)
				m_ServoMotorValue3.data = m_ServoMotor3LimitMin;
		}

		m_ServoMotorValue2Out.write();
		m_ServoMotorValue3Out.write();
	}
	if (butonFlag == true)
	{
		if (button[3] == 1){//{^Ăꍇ
			m_ServoMotorValue1.data += m_ServoMotor1ChangeValue;//T[{[^3̊px
			if (m_ServoMotorValue1.data > m_ServoMotor1LimitMax)
				m_ServoMotorValue1.data = m_ServoMotor1LimitMax;
		}
		if (button[2] == 1){//{^Ăꍇ
			m_ServoMotorValue1.data -= m_ServoMotor1ChangeValue;//T[{[^3̊px
			if (m_ServoMotorValue1.data < m_ServoMotor1LimitMin)
				m_ServoMotorValue1.data = m_ServoMotor1LimitMin;
		}
		m_ServoMotorValue1Out.write();
	}

}

/*
RTC::ReturnCode_t ConvertToMotor::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ConvertToMotor::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ConvertToMotor::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ConvertToMotor::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ConvertToMotor::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ConvertToMotorInit(RTC::Manager* manager)
  {
    coil::Properties profile(converttomotor_spec);
    manager->registerFactory(profile,
                             RTC::Create<ConvertToMotor>,
                             RTC::Delete<ConvertToMotor>);
  }
  
};


